# -*- project-name: VASM -*-
package VASM::Action;

use strict;
use warnings;
use XML::Parser;
use XML::Writer;

our $VERSION = '1.04';

sub new {
  my ($self, $handle) = @_;

  my $instance = []; # Commands pushed and popped using arrayref
  bless $instance, $self;

  # Initialize the instance if a file is specified
  $instance->Parse($handle) if defined $handle;

  return $instance;
}

# Add another command string to the end of the actions list
sub Push {
  my ($self, $command) = @_;

  push @{ $self }, $command;
  return;
}

# You get the idea...
sub Unshift {
  my ($self, $command) = @_;
  
  unshift @{ $self }, $command;
  return;
}

# Remove a command string from the end of the actions list
sub Pop {
  my ($self) = @_;
  
  pop @{ $self };
  return;
}

sub Shift {
  my ($self) = @_;
  
  shift @{ $self };
  return;
}

# Just return the contents of the instance
sub Dump {
  my ($self) = @_;
  
  return @{ $self };
}

# Clear out the catalog altogether
sub Clear {
  my ($self) = @_;
  
  $#{ $self } = -1;
  
  return;
}

sub Exec {
  my ($self, @arguments) = @_; my $status;
  
  for my $program (@{ $self }) {
    $status = system sprintf $program, @arguments;
  }

  return $status; # Will be -1 if none found
}

sub Parse {
  my ($self, $handle) = @_;
  # If this fails, XML::Parser will emit an error message and end the program.
  # Unfortunately, this message is always English. :( There is at least an
  # error context, which can be universally understood by the programmer.
  my $parser = XML::Parser->new(Style => 'Tree', ErrorContext => 3);
  my $tree = $parser->parse($handle); my $tags = $tree->[1];
  
  # Iterate through tags under <actions>
  for my $index (grep { $_ % 2 } (0..$#{ $tags })) {
    next if $tags->[$index] eq '0'; # Skip text elements 

    # Otherwise, it /has to be/ a command string. Return if it does not
    # contain a text string or holds no content
    return unless $tags->[$index + 1]->[1] eq '0'
      and defined $tags->[$index + 1]->[2];

    # Push the new member then
    $self->Push($tags->[$index + 1]->[2]);
  }

  return 1; # Success!
}

# Write out the contents of the instance to a file, given a filehandle
sub Write {
  my ($self, $handle) = @_;

  # Now create an XML::Writer instance and turn that puppy out
  my $xml = XML::Writer->new(OUTPUT => $handle, ENCODING => 'utf-8',
                             DATA_MODE => 1, DATA_INDENT => 2);

  # Open up the document
  $xml->xmlDecl;
  $xml->startTag('actions');
  $xml->comment('Generated by VASM; hanumizzle was here'); # Had to... :D

  # Emit the 'command' tags
  for my $command (@{ $self }) {
    $xml->dataElement('command', $command);
  }

  $xml->endTag('actions'); $xml->end; # End the document...

  return 1; # Successful!
}
  
1;

__END__

=head1 NAME

VASM::Action - preference registration for diverse media types

=head1 SYNOPSIS

    use VASM::Action;
    
    # Instantiate VASM::Action object using an IO::Handle. This will call the
    # Parse method of the new instance, passing that handle in.
    my $instance = VASM::Action->new($ioHandle);
    
    # Wipe out what was there earlier
    $instance->Clear;
    
    # Add some new programs
    $instance->Push('gvim %s');
    $instance->Push('nedit %s');
    $instance->Push('notepad %s');
    
    # Change your mind?
    $instance->Pop;
    
    # Even better
    $instance->Unshift('emacsclient %s');
    
    # (Push, Pop, Unshift, and Shift are all available, although they return
    # nothing)
    
    # Print out contents of catalog
    print for $instance->Dump;

    # Execute the first command found in the catalog with an argument. This
    # will run 'emacsclient /home/hanuman/sithon-and-manola'
    $instance->Exec('/home/hanuman/sithon-and-manola');
    
    # Write out the contents of the catalog via IO::Handle 
    $instance->Write($ioHandle);

=head1 DESCRIPTION

VASM::Action defines lists of programs associated with a given media type as
viewers and/or editors. In principle, VASM::Action can be used with any media
type represented in the arguments to a Unix command, as it is completely
agnostic towards them, being responsible only for executing a command in a
list. In practice, this means action catalogs are used in conjunction with
VASM::Resource::Action to pair vanilla file pathnames and URIs (Uniform
Resource Indicators, such as http://distro.ibiblio.org) to preferred programs.
Once prepared, the Exec method of an instance will systematically run through
a list of programs and execute the first valid command with arguments, if so
desired. Exec may fail to find a single usable command line; in this case, it
will return -1, propagating the return value of Perl's built-in system
function (as of Perl 5.x; this is liable to change in Perl 6).

=head1 METHODS

=over 4

=item new

=item Push, Pop, Unshift, and Shift

=item Dump

=item Clear

=item Parse

=item Write

=item Exec

=back

=head1 AUTHORS

hanumizzle L<hanumizzle@gmail.com> wrote VASM::Action.

=cut
